#
#      Tokyoflash-Uhr: Stencil
#      -----------------------
#         Uwe Berger; 2012
#
# http://www.tokyoflash.com/de/watches/kisai/stencil/
#

# Groesse einer Einheit
set gvar(dx) 60
set gvar(dy) $gvar(dx)

# Linienstarke
set gvar(line_width) 6

# Hintergrundfarbe
set gvar(bg_color) darkblue

# Linienfarbe
set gvar(line_color) white

# Darstellung mit/ohne Sekunden
set gvar(with_sec) 0

# interne Variable
set gvar(old_s) 60
set gvar(old_m) 60
set gvar(old_h) 25

# Digit-Definition 0...9
# Linienelement jeweils x0 y0 x1 y1
set digit [list\
		{{1 1 1 2}}\
		{{1 0 1 3}}\
		{{0 1 1 1} {1 2 2 2}}\
		{{0 1 1 1} {0 2 1 2}}\
		{{1 0 1 1} {0 2 1 2} {1 2 1 3}}\
		{{1 1 2 1} {0 2 1 2}}\
		{{1 1 2 1} {1 2 1 2}}\
		{{0 1 1 1} {1 1 1 3}}\
		{{1 1 1 1} {1 2 1 2}}\
		{{1 1 1 1} {0 2 1 2}}\
	]
	
#***************************
proc gui {} {
	global gvar
	# Fenster generieren
	wm title . "Tokyoflash Stencil" 
	wm resizable . 0 0
	set wdx [expr $gvar(dx) * 4]
	if {$gvar(with_sec)} {
		set wdy [expr $gvar(dy) * 9]
		set ly1 [expr 9 * $gvar(dy)]
	} else {
		set wdy [expr $gvar(dy) * 6]
		set ly1 [expr 6 * $gvar(dy)]
	}
	canvas	.clock -width $wdx -height $wdy -bg $gvar(bg_color) -bd 0
	pack .clock 
	# die zwei Grundlinien zeichnen ({0 3 4 3}{2 0 2 6})
	.clock create line 	[expr 0 * $gvar(dx)]\
						[expr 3 * $gvar(dy)]\
						[expr 4 * $gvar(dx)]\
						[expr 3 * $gvar(dy)]\
						-width $gvar(line_width)\
						-fill $gvar(line_color)\
						-capstyle projecting
	.clock create line 	[expr 2 * $gvar(dx)]\
						[expr 0 * $gvar(dy)]\
						[expr 2 * $gvar(dx)]\
						$ly1\
						-width $gvar(line_width)\
						-fill $gvar(line_color)\
						-capstyle projecting
	if {$gvar(with_sec)} {
		.clock create line 	[expr 0 * $gvar(dx)]\
							[expr 6 * $gvar(dy)]\
							[expr 4 * $gvar(dx)]\
							[expr 6 * $gvar(dy)]\
							-width $gvar(line_width)\
							-fill $gvar(line_color)\
							-capstyle projecting
	}
}

#***************************
proc draw_digit {val offset_x offset_y} {
	global gvar digit
	.clock delete d$offset_x$offset_y
	foreach line [lindex $digit $val] {
		foreach {x0 y0 x1 y1} $line {
			# wenn Punkte gezeichnet werden sollen,
			# Endpunkt + Linienbreite in X-Richtung hinzurechnen
			set ox 0
			if {([expr $x0 - $x1] == 0) && ([expr $y0 - $y1] == 0)} {
				set ox $gvar(line_width)
			}
			.clock create line 	[expr ($x0 * $gvar(dx)) + ($offset_x * $gvar(dx))]\
								[expr ($y0 * $gvar(dy)) + ($offset_y * $gvar(dy))]\
								[expr ($x1 * $gvar(dx)) + ($offset_x * $gvar(dx)) + $ox/2]\
								[expr ($y1 * $gvar(dy)) + ($offset_y * $gvar(dy))]\
								-width $gvar(line_width)\
								-fill $gvar(line_color)\
								-capstyle projecting\
								-tags d$offset_x$offset_y
		}
	}
}

#***************************
proc takt {} {
	global gvar
	# Uhrzeit ermitteln
	set m [scan [clock format [clock seconds] -format %M] %d]
	set h [scan [clock format [clock seconds] -format %H] %d]
	set s [scan [clock format [clock seconds] -format %S] %d]
	# hat sich etwas an der Zeit geaendert...
	# ...Sekunden
	if {($gvar(old_s) != $s) && ($gvar(with_sec))} {
		set gvar(old_s) $s
		# Sekunde-Zehner
		draw_digit [expr $s/10] 0 6
		# Sekunde-Einer
		draw_digit [expr $s%10] 2 6
	}
	# ...Minuten
	if {$gvar(old_m) != $m} {
		set gvar(old_m) $m
		# Minute-Zehner
		draw_digit [expr $m/10] 0 3
		# Minute-Einer
		draw_digit [expr $m%10] 2 3
	}
	# ...Stunden
	if {$gvar(old_h) != $h} {
		set gvar(old_h) $h
		# Stunde-Zehner
		draw_digit [expr $h/10] 0 0
		# Stunde-Einer
		draw_digit [expr $h%10] 2 0
	}
	# Prozedur nach 1 Sekunde neu aufrufen	
	after 1000 takt
}

#***************************
#***************************
#***************************
gui
takt


		
